classdef encif < handle
    %ENCIF Encoder Interface
    %   Rotational encoder at serial port sending data telegrams containing timestamp and value  
    %   tested with Matlab R2014a, R2014b, R2017a   
    %
    % 14.03.2019 Nicolas
    
    properties (SetAccess = private)
        ticksPerRevolution;
        SerialObj;
        Rawdata;
        idx;
        prevRead;
    end
    properties (Dependent = true)
        time_s;    % time in seconds
        angle_deg; % angle in degrees
        speed_rpm; % Speed in revolution per seconds
    end
    
    
    methods
        function self = encif(varargin)
            % Constructor opens serial port 
            
            % Interprete arguments
            S = struct(varargin{:});
            
            if( isfield(S, 'serialPort') )
                serialPort = S.serialPort;
            else
                error('missing input paramater: ''%s'' ', 'serialPort');
            end

            if( isfield(S, 'ticksPerRevolution') )
                self.ticksPerRevolution = S.ticksPerRevolution;
            else
                error('missing input paramater: ''%s'' ', 'ticksPerRevolution');
            end
            
            % Check if COM is opened in Matlab and close if 
            ExistingSerialPorts = instrfind('Port', serialPort);
            if( ~isempty(ExistingSerialPorts) )
                fclose(ExistingSerialPorts);
            end

            % Serial port (with callback function)
            self.SerialObj = serial(serialPort, 'Terminator', 'CR/LF');
            self.SerialObj.BytesAvailableFcnMode = 'terminator';
            self.SerialObj.BytesAvailableFcn = @(obj, event) getFromSerial(self);

            % Initialize data
            self.Rawdata = zeros(3, 1000);
            self.idx = 1;
            self.prevRead = [NaN; NaN];
        end
        
        function start(self)
            fopen(self.SerialObj);
        end
        
        function stop(self)
            fclose(self.SerialObj);
        end
       
        
        function getFromSerial(self)
            % Get Data from serial port, update position and speed and store everything as raw data
            
            % Get data from serial port (time and value)
            % T=0x1234, V=0x5678\n   
            % Timestamp in milliseconds, encoder value in ticks
            d = fscanf(self.SerialObj, 'T=0x%x, V=0x%x\n');
            t_u16 = d(1);
            u_u16 = d(2);

            
            if( isnan(self.prevRead(1)) )
                % Discard very first read, since there exists no difference
                self.prevRead = [t_u16; u_u16];
                return;
            else
                % Calc increment
                pt_u16 = self.prevRead(1);
                pu_u16 = self.prevRead(2);
                
                % Do some correct 16-Bit integer overflow
                dt = encif.diff_u16(t_u16, pt_u16);
                du = encif.diff_u16(u_u16, pu_u16);
            
                self.prevRead = [t_u16; u_u16];
            end

            % Old values from previous step
            last = self.Rawdata(:, self.idx);

            % accumulate
            time = last(1)+ dt;
            value = last(2) + du;
            speed = du/dt;            
            
            % Update            
            this = [time; value; speed];
            self.idx = self.idx + 1;
            self.Rawdata(:, self.idx) = this;
            self.idx = self.idx;

            % Preallocation
            blockSize = 1000;
            if( mod(self.idx, blockSize) == 0 )
                self.Rawdata(:, self.idx + blockSize) = [0; 0; 0];
            end
        end
    end
    
    
    %% Getter functions
    methods
        function out = get.time_s(self)
            % time in seconds, is stored as milliseconds
            out = self.Rawdata(1, 1:self.idx)/1000;
        end
        
        function out = get.angle_deg(self)
            % angle in degrees, is stored as encoder ticks
            out = self.Rawdata(2, 1:self.idx);
            out = out * 360/self.ticksPerRevolution;
        end
        
        function out = get.speed_rpm(self)
            % Speed in revolutiosn per minute, is stored as ticks per 
            % millisecond
            out = self.Rawdata(3, 1:self.idx);
            out = out * 60000/self.ticksPerRevolution;
        end
    end
    
    
    methods (Static = true)
        function d = diff_u16(a, b)
            %% Difference of two unsigned integer with correct sign
            % the variables a and b do not need to be of Matlab class uint16,
            % but overflow of this type is considered
            %
            % 08.03.2019 Nicolas
            
            if( a > b )
                d1 = a-b;
                d2 = a - b - 2^16;
            else
                d1 = a-b;
                d2 = a + 2^16 - b;
            end
            
            if( abs(d1) < abs(d2) )
                d = d1;
            else
                d = d2;
            end
            
        end
    end
    
    
end

